<?php

namespace Moblyze\FinancialPlan\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Moblyze\FinancialPlan\Http\Requests\FinancialPlanRequest;
use Moblyze\FinancialPlan\Interfaces\FinancialPlanRepositoryInterface;
use Moblyze\FinancialPlan\Models\FinancialPlan;
use Moblyze\FinancialPlan\Forms\FinancialPlan as Form;

/**
 * APIController  class for financial_plan.
 */
class FinancialPlanAPIController extends BaseController
{

    /**
     * Initialize financial_plan resource controller.
     *
     * @param type FinancialPlanRepositoryInterface $financial_plan
     *
     * @return null
     */
    public function __construct(FinancialPlanRepositoryInterface $financial_plan)
    {
        parent::__construct();
        $this->repository = $financial_plan;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moblyze\FinancialPlan\Repositories\Criteria\FinancialPlanResourceCriteria::class);
    }

    /**
     * Display a list of financial_plan.
     *
     * @return Response
     */
    public function index(FinancialPlanRequest $request)
    {
        return $this->repository
            ->setPresenter(\Moblyze\FinancialPlan\Repositories\Presenter\FinancialPlanPresenter::class)
            ->paginate();
    }

    /**
     * Display financial_plan.
     *
     * @param Request $request
     * @param Model   $financial_plan
     *
     * @return Response
     */
    public function show(FinancialPlanRequest $request, FinancialPlan $financial_plan)
    {
        return $financial_plan->setPresenter(\Moblyze\FinancialPlan\Repositories\Presenter\FinancialPlanListPresenter::class);
        ;
    }

    /**
     * Create new financial_plan.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FinancialPlanRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('financial_plan::financial_plan.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('financial_plan/financial_plan/' . $financial_plan->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('financial_plan/financial_plan');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the financial_plan.
     *
     * @param Request $request
     * @param Model   $financial_plan
     *
     * @return Response
     */
    public function update(FinancialPlanRequest $request, FinancialPlan $financial_plan)
    {
        try {
            $data = $request->all();

            $financial_plan->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('financial_plan::financial_plan.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('financial_plan/financial_plan/' . $financial_plan->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('financial_plan/financial_plan/' . $financial_plan->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the financial_plan.
     *
     * @param Model   $financial_plan
     *
     * @return Response
     */
    public function destroy(FinancialPlanRequest $request, FinancialPlan $financial_plan)
    {
        try {
            $financial_plan->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('financial_plan::financial_plan.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('financial_plan/financial_plan/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('financial_plan/financial_plan/' . $financial_plan->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
