<?php

namespace Moblyze\FinancialPlan\Policies;

use Litepie\User\Contracts\UserPolicy;
use Moblyze\FinancialPlan\Models\FinancialPlan;

class FinancialPlanPolicy
{

    /**
     * Determine if the given user can view the financial_plan.
     *
     * @param UserPolicy $user
     * @param FinancialPlan $financial_plan
     *
     * @return bool
     */
    public function view(UserPolicy $user, FinancialPlan $financial_plan)
    {
        if ($user->canDo('financial_plan.financial_plan.view') && $user->isAdmin()) {
            return true;
        }

        return $financial_plan->user_id == user_id() && $financial_plan->user_type == user_type();
    }

    /**
     * Determine if the given user can create a financial_plan.
     *
     * @param UserPolicy $user
     * @param FinancialPlan $financial_plan
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('financial_plan.financial_plan.create');
    }

    /**
     * Determine if the given user can update the given financial_plan.
     *
     * @param UserPolicy $user
     * @param FinancialPlan $financial_plan
     *
     * @return bool
     */
    public function update(UserPolicy $user, FinancialPlan $financial_plan)
    {
        if ($user->canDo('financial_plan.financial_plan.edit') && $user->isAdmin()) {
            return true;
        }

        return $financial_plan->user_id == user_id() && $financial_plan->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given financial_plan.
     *
     * @param UserPolicy $user
     * @param FinancialPlan $financial_plan
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, FinancialPlan $financial_plan)
    {
        return $financial_plan->user_id == user_id() && $financial_plan->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given financial_plan.
     *
     * @param UserPolicy $user
     * @param FinancialPlan $financial_plan
     *
     * @return bool
     */
    public function verify(UserPolicy $user, FinancialPlan $financial_plan)
    {
        if ($user->canDo('financial_plan.financial_plan.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given financial_plan.
     *
     * @param UserPolicy $user
     * @param FinancialPlan $financial_plan
     *
     * @return bool
     */
    public function approve(UserPolicy $user, FinancialPlan $financial_plan)
    {
        if ($user->canDo('financial_plan.financial_plan.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
